@@ -1,8 +1,8 @@
 //! `NcCell` methods and associated functions.
 
 use crate::{
-    cstring, error, nccell_load, NcAlphaBits, NcCell, NcChannels, NcComponent, NcEgcBackstop,
-    NcPaletteIndex, NcPlane, NcResult, NcRgb, NcStyle, NCRESULT_ERR,
+    cstring, error, nccell_load, rstring, NcAlphaBits, NcCell, NcChannels, NcComponent,
+    NcEgcBackstop, NcError, NcPaletteIndex, NcPlane, NcResult, NcRgb, NcStyle, NCRESULT_ERR,
 };
 
 #[allow(unused_imports)] // for the doc comments
@@ -13,38 +13,49 @@ impl NcCell {
     /// New `NcCell`, expects a 7-bit [`char`].
     #[inline]
     #[allow(clippy::unnecessary_cast)]
-    pub const fn from_char7b(ch: char) -> Self {
-        NcCell {
+    pub fn from_char7b(ch: char) -> NcResult<Self> {
+        if !ch.is_ascii() {
+            return Err(NcError::new());
+        }
+        Ok(NcCell {
             gcluster: (ch as u32).to_le(),
             gcluster_backstop: 0 as NcEgcBackstop,
             width: 0_u8,
             stylemask: 0 as NcStyle,
             channels: 0 as NcChannels,
+        })
+    }
+
+    /// New `NcCell`, from a [`char`].
+    ///
+    /// Expects a plane where to save the extra data if it's greater than 4 bytes.
+    #[inline]
+    pub fn from_char(plane: &mut NcPlane, ch: char) -> NcResult<Self> {
+        let mut cell = Self::new();
+        let res = unsafe { nccell_load(plane, &mut cell, cstring![ch.to_string()]) };
+        if res == NCRESULT_ERR {
+            return Err(NcError::new());
         }
+        Ok(cell)
     }
 
-    /// New `NcCell`, expects an [`NcPlane`] and a [`char`].
+    /// New `NcCell`, from a [`&str`].
+    ///
+    /// Expects a plane where to save the extra data if it's greater than 4 bytes.
     #[inline]
-    pub fn from_char(ch: char, plane: &mut NcPlane) -> Self {
+    pub fn from_str(plane: &mut NcPlane, string: &str) -> NcResult<Self> {
         let mut cell = Self::new();
-        let result = unsafe { nccell_load(plane, &mut cell, cstring![ch.to_string()]) };
-        debug_assert_ne![NCRESULT_ERR, result];
-        cell
-    }
-
-    /// New `NcCell`, expects an [`NcPlane`] and a &[`str`].
-    #[inline]
-    pub fn from_str(plane: &mut NcPlane, string: &str) -> Self {
-        let mut cell = Self::new();
-        let result = unsafe { nccell_load(plane, &mut cell, cstring![string]) };
-        debug_assert_ne![NCRESULT_ERR, result];
-        cell
+        let res = unsafe { nccell_load(plane, &mut cell, cstring![string]) };
+        if res == NCRESULT_ERR {
+            return Err(NcError::new());
+        }
+        Ok(cell)
     }
 
     /// New empty `NcCell`.
     #[inline]
-    pub const fn new() -> Self {
-        Self::from_char7b(0 as char)
+    pub fn new() -> Self {
+        Self::from_char7b(0 as char).unwrap()
     }
 
     /// Breaks the UTF-8 string in `egc` down, setting up this `NcCell`,
@@ -86,8 +97,10 @@ impl NcCell {
     /// Both must be or will be bound to `common_plane`.
     ///
     /// *C style function: [nccell_duplicate()][crate::nccell_duplicate].*
-    pub fn duplicate(&self, target: &mut NcCell, common_plane: &mut NcPlane) -> NcResult<()> {
-        error![unsafe { crate::nccell_duplicate(common_plane, target, self) }]
+    pub fn duplicate(&self, common_plane: &mut NcPlane) -> NcResult<NcCell> {
+        let mut target = NcCell::new();
+        let res = unsafe { crate::nccell_duplicate(common_plane, &mut target, self) };
+        error![res, "NcCell.duplicate()", target]
     }
 
     /// Initializes (zeroes out) this `NcCell`.
@@ -116,7 +129,7 @@ impl NcCell {
     /// *(No equivalent C style function)*
     pub fn channels(&mut self, plane: &mut NcPlane) -> NcChannels {
         let (mut _styles, mut channels) = (0, 0);
-        let _char = crate::nccell_extract(plane, self, &mut _styles, &mut channels);
+        let _egc = crate::nccell_extract(plane, self, &mut _styles, &mut channels);
         channels
     }
 
@@ -358,16 +371,25 @@ impl NcCell {
 
 /// # `NcCell` methods: text
 impl NcCell {
-    // /// Returns a pointer to the `EGC` of this NcCell in the [NcPlane] `plane`.
-    // ///
-    // /// This pointer can be invalidated by any further operation on the referred
-    // /// plane, so… watch out!
-    // ///
-    // /// *C style function: [nccell_extended_gcluster()][crate::nccell_wide_left_p].*
-    // pub fn extended_gcluster(&self, plane: &NcPlane) -> u32 {
-    //     let egcpointer = unsafe { crate::nccell_extended_gcluster(plane, self) };
-    //     egcpointer
-    // }
+    /// Returns the number of columns occupied by the cell.
+    ///
+    /// See [`ncstrwidth`][crate::ncstrwidth] for an equivalent for multiple EGCs.
+    ///
+    /// *C style function: [nccell_cols()][crate::nccell_cols].*
+    pub const fn cols(&self) -> u8 {
+        crate::nccell_cols(self)
+    }
+
+    /// Returns a pointer to the `EGC` of this NcCell in the `plane`.
+    ///
+    /// This pointer can be invalidated by any further operation on the referred
+    /// plane, so… watch out!
+    ///
+    /// *C style function: [nccell_extended_gcluster()][crate::nccell_wide_left_p].*
+    pub fn extended_gcluster(&self, plane: &NcPlane) -> &str {
+        let egcpointer = unsafe { crate::nccell_extended_gcluster(plane, self) };
+        rstring![egcpointer]
+    }
 
     /// Copies the UTF8-encoded `EGC` out of this NcCell,
     /// whether simple or complex.
@@ -465,4 +487,62 @@ impl NcCell {
             crate::nccells_rounded_box(plane, style as u32, channels, ul, ur, ll, lr, hl, vl)
         }]
     }
+
+    /// NcCell.[load_box()][NcCell#method.box] with ASCII characters.
+    ///
+    /// *C style function: [nccells_ascii_box()][crate::nccells_ascii_box].*
+    pub fn ascii_box(
+        plane: &mut NcPlane,
+        style: NcStyle,
+        channels: NcChannels,
+        ul: &mut NcCell,
+        ur: &mut NcCell,
+        ll: &mut NcCell,
+        lr: &mut NcCell,
+        hl: &mut NcCell,
+        vl: &mut NcCell,
+    ) -> NcResult<()> {
+        error![crate::nccells_ascii_box(
+            plane, style, channels, ul, ur, ll, lr, hl, vl
+        )]
+    }
+    /// NcCell.[load_box()][NcCell#method.box] with the heavy line
+    /// box-drawing characters.
+    ///
+    /// *C style function: [nccells_heavy_box()][crate::nccells_heavy_box].*
+    pub fn heavy_box(
+        plane: &mut NcPlane,
+        style: NcStyle,
+        channels: NcChannels,
+        ul: &mut NcCell,
+        ur: &mut NcCell,
+        ll: &mut NcCell,
+        lr: &mut NcCell,
+        hl: &mut NcCell,
+        vl: &mut NcCell,
+    ) -> NcResult<()> {
+        error![crate::nccells_heavy_box(
+            plane, style, channels, ul, ur, ll, lr, hl, vl
+        )]
+    }
+
+    /// NcCell.[load_box()][NcCell#method.box] with the light line
+    /// box-drawing characters.
+    ///
+    /// *C style function: [nccells_light_box()][crate::nccells_light_box].*
+    pub fn light_box(
+        plane: &mut NcPlane,
+        style: NcStyle,
+        channels: NcChannels,
+        ul: &mut NcCell,
+        ur: &mut NcCell,
+        ll: &mut NcCell,
+        lr: &mut NcCell,
+        hl: &mut NcCell,
+        vl: &mut NcCell,
+    ) -> NcResult<()> {
+        error![crate::nccells_light_box(
+            plane, style, channels, ul, ur, ll, lr, hl, vl
+        )]
+    }
 }
