@@ -1,5 +1,7 @@
 //! `cell*_*` reimplemented functions.
 
+#![allow(dead_code)]
+
 use libc::strcmp;
 
 use crate::{
@@ -9,6 +11,13 @@ use crate::{
     NCRESULT_OK, NCSTYLE_MASK,
 };
 
+const NCBOXLIGHT: &str = "┌┐└┘─│";
+const NCBOXHEAVY: &str = "┏┓┗┛━┃";
+const NCBOXROUND: &str = "╭╮╰╯─│";
+const NCBOXDOUBLE: &str = "╔╗╚╝═║";
+const NCBOXASCII: &str = "/\\\\/-|";
+const NCBOXOUTER: &str = "🭽🭾🭼🭿▁🭵🭶🭰";
+
 // Alpha -----------------------------------------------------------------------
 
 /// Extracts the foreground [`NcAlphaBits`] from an [`NcCell`] (shifted to LSBs).
@@ -221,9 +230,9 @@ pub const fn nccell_bg_palindex(cell: &NcCell) -> NcPaletteIndex {
 /// Also sets [NCALPHA_FG_PALETTE] and [NCALPHA_OPAQUE],
 /// and clears out [NCALPHA_FGDEFAULT_MASK].
 ///
+/// NOTE: Unlike the original C function, this one can't fail.
+///
 /// *Method: NcCell.[set_fg_palindex()][NcCell#method.set_fg_palindex].*
-//
-// NOTE: unlike the original C function, this one can't fail
 #[inline]
 #[allow(clippy::unnecessary_cast)]
 pub fn nccell_set_fg_palindex(cell: &mut NcCell, index: NcPaletteIndex) {
@@ -239,9 +248,9 @@ pub fn nccell_set_fg_palindex(cell: &mut NcCell, index: NcPaletteIndex) {
 /// Also sets [`NCALPHA_BG_PALETTE`] and [`NCALPHA_OPAQUE`],
 /// and clears out [`NCALPHA_BGDEFAULT_MASK`].
 ///
+/// NOTE: Unlike the original C function, this one can't fail.
+///
 /// *Method: NcCell.[set_bg_palindex()][NcCell#method.set_bg_palindex].*
-//
-// NOTE: unlike the original C function, this one can't fail
 #[inline]
 pub fn nccell_set_bg_palindex(cell: &mut NcCell, index: NcPaletteIndex) {
     cell.channels |= NCALPHA_BGDEFAULT_MASK as NcChannels;
@@ -289,6 +298,23 @@ pub fn nccell_set_styles(cell: &mut NcCell, stylebits: NcStyle) {
 
 // Chars -----------------------------------------------------------------------
 
+/// Returns the number of columns occupied by `cell`.
+///
+/// See [`ncstrwidth`][crate::ncstrwidth] for an equivalent for multiple EGCs.
+#[inline]
+pub const fn nccell_cols(cell: &NcCell) -> u8 {
+    if cell.width != 0 {
+        cell.width
+    } else {
+        1
+    }
+}
+
+#[deprecated]
+pub fn nccell_width(plane: &NcPlane, cell: &NcCell) -> NcIntResult {
+    unsafe { crate::ffi::nccell_width(plane, cell) }
+}
+
 /// Does the [`NcCell`] contain an East Asian Wide codepoint?
 ///
 /// *Method: NcCell.[double_wide_p()][NcCell#method.double_wide_p].*
@@ -313,47 +339,6 @@ pub const fn nccell_wide_left_p(cell: &NcCell) -> bool {
     nccell_double_wide_p(cell) && cell.gcluster != 0
 }
 
-// /// Loads a 7-bit `EGC` character into the [`NcCell`].
-// ///
-// /// *Method: NcCell.[load_char()][NcCell#method.load_char].*
-// //
-// // TODO:CHECK is this necessary at all?
-// #[inline]
-// pub fn nccell_load_char(plane: &mut NcPlane, cell: &mut NcCell, ch: char) /* -> i32 */
-// {
-//     let _ = unsafe { crate::nccell_load(plane, cell, ch) };
-// }
-// nccell_load_char(struct ncplane* n, nccell* c, char ch){
-//   char gcluster[2];
-//   gcluster[0] = ch;
-//   gcluster[1] = '\0';
-//   let _ = nccell_load(n, c, gcluster);
-// }
-
-// /// Loads a UTF-8 grapheme cluster of up to 4 bytes into the cell `c`.
-// ///
-// /// *Method: NcCell.[load_egc32()][NcCell#method.load_egc32].*
-// //
-// // TODO
-// #[inline]
-// pub fn nccell_load_egc32(plane: &mut NcPlane, cell: &mut NcCell, egc: &str) -> NcIntResult {
-//     char gcluster[sizeof(egc) + 1];
-//     egc = egc.to_le();
-//     memcpy(gcluster, &egc, sizeof(egc));
-//     gcluster[4] = '\0';
-//     return nccell_load(n, c, gcluster);
-// }
-// // Load a UTF-8 encoded EGC of up to 4 bytes into the nccell 'c'. Returns the
-// // number of bytes used, or -1 on error.
-// static inline int
-// nccell_load_egc32(struct ncplane* n, nccell* c, uint32_t egc){
-//   char gcluster[sizeof(egc) + 1];
-//   egc = htole(egc);
-//   memcpy(gcluster, &egc, sizeof(egc));
-//   gcluster[4] = '\0';
-//   return nccell_load(n, c, gcluster);
-// }
-
 /// Copies the UTF8-encoded `EGC` out of the [`NcCell`], whether simple or complex.
 ///
 /// The result is not tied to the [NcPlane],
@@ -378,12 +363,8 @@ pub fn nccell_extract(
     stylemask: &mut NcStyle,
     channels: &mut NcChannels,
 ) -> String {
-    if *stylemask != 0 {
-        *stylemask = cell.stylemask;
-    }
-    if *channels != 0 {
-        *channels = cell.channels;
-    }
+    *stylemask = cell.stylemask;
+    *channels = cell.channels;
     nccell_strdup(plane, cell)
 }
 
@@ -428,6 +409,7 @@ pub fn nccell_init(cell: &mut NcCell) {
 /// - Blasts the styling with `style` and `channels`.
 ///
 /// *Method: NcCell.[prime()][NcCell#method.prime].*
+#[inline]
 pub fn nccell_prime(
     plane: &mut NcPlane,
     cell: &mut NcCell,
@@ -448,6 +430,7 @@ pub fn nccell_prime(
 /// are [nccell_release]d. There must be at least six `EGC`s in `gcluster`.
 ///
 /// *Method: NcCell.[load_box()][NcCell#method.load_box].*
+#[inline]
 pub fn nccells_load_box(
     plane: &mut NcPlane,
     style: NcStyle,
@@ -514,3 +497,57 @@ pub fn nccells_load_box(
     }
     NCRESULT_ERR
 }
+
+/// [nccells_load_box] with ASCII characters.
+///
+/// *Method: NcCell.[ascii_box()][NcCell#method.ascii_box].*
+#[inline]
+pub fn nccells_ascii_box(
+    plane: &mut NcPlane,
+    style: NcStyle,
+    channels: NcChannels,
+    ul: &mut NcCell,
+    ur: &mut NcCell,
+    ll: &mut NcCell,
+    lr: &mut NcCell,
+    hl: &mut NcCell,
+    vl: &mut NcCell,
+) -> NcIntResult {
+    nccells_load_box(plane, style, channels, ul, ur, ll, lr, hl, vl, NCBOXASCII)
+}
+
+/// [nccells_load_box] with heavy line box-drawing characters.
+///
+/// *Method: NcCell.[heavy_box()][NcCell#method.heavy_box].*
+#[inline]
+pub fn nccells_heavy_box(
+    plane: &mut NcPlane,
+    style: NcStyle,
+    channels: NcChannels,
+    ul: &mut NcCell,
+    ur: &mut NcCell,
+    ll: &mut NcCell,
+    lr: &mut NcCell,
+    hl: &mut NcCell,
+    vl: &mut NcCell,
+) -> NcIntResult {
+    nccells_load_box(plane, style, channels, ul, ur, ll, lr, hl, vl, NCBOXHEAVY)
+}
+
+/// [nccells_load_box] with light line box-drawing characters.
+///
+/// *Method: NcCell.[light_box()][NcCell#method.light_box].*
+#[inline]
+pub fn nccells_light_box(
+    plane: &mut NcPlane,
+    style: NcStyle,
+    channels: NcChannels,
+    ul: &mut NcCell,
+    ur: &mut NcCell,
+    ll: &mut NcCell,
+    lr: &mut NcCell,
+    hl: &mut NcCell,
+    vl: &mut NcCell,
+) -> NcIntResult {
+    nccells_load_box(plane, style, channels, ul, ur, ll, lr, hl, vl, NCBOXLIGHT)
+}
