@@ -3,7 +3,7 @@
 use core::ptr::{null, null_mut};
 
 use crate::{
-    cstring, error, error_ref_mut, rstring, NcAlign, NcBlitter, NcCapabilities, NcChannels,
+    cstring, error, error_ref_mut, rstring_free, NcAlign, NcBlitter, NcCapabilities, NcChannels,
     NcComponent, NcDim, NcDirect, NcDirectFlags, NcError, NcInput, NcOffset, NcPaletteIndex,
     NcPlane, NcResult, NcRgb, NcScale, NcStyle, NcTime, NCRESULT_ERR,
 };
@@ -232,6 +232,13 @@ impl NcDirect {
         ]
     }
 
+    /// Returns the current styling.
+    ///
+    /// *C style function: [ncdirect_styles()][crate::ncdirect_styles].*
+    pub fn styles(&self) -> NcStyle {
+        unsafe { crate::ncdirect_styles(self) as NcStyle }
+    }
+
     /// Removes the specified styles.
     ///
     /// *C style function: [ncdirect_off_styles()][crate::ncdirect_off_styles].*
@@ -262,6 +269,18 @@ impl NcDirect {
         ]
     }
 
+    /// Returns an [`NcStyle`] with the supported curses-style attributes.
+    ///
+    /// The attribute is only indicated as supported if the terminal can support
+    /// it together with color.
+    ///
+    /// For more information, see the "ncv" capability in terminfo(5).
+    ///
+    /// *C style function: [ncdirect_supported_styles()][crate::ncdirect_supported_styles].*
+    pub fn supported_styles(&self) -> NcStyle {
+        unsafe { crate::ncdirect_supported_styles(self) as NcStyle }
+    }
+
     /// Indicates to use the "default color" for the foreground.
     ///
     /// *C style function: [ncdirect_set_fg_default()][crate::ncdirect_set_fg_default].*
@@ -541,12 +560,18 @@ impl NcDirect {
     ///
     /// *C style function: [ncdirect_detected_terminal()][crate::ncdirect_detected_terminal].*
     pub fn detected_terminal(&self) -> String {
-        rstring![crate::ncdirect_detected_terminal(self)].to_string()
+        rstring_free![crate::ncdirect_detected_terminal(self)]
     }
 }
 
 /// ## NcDirect methods: I/O
 impl NcDirect {
+    #[doc(hidden)]
+    #[deprecated = "use `get` method instead"]
+    pub fn getc(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
+        self.get(time, input)
+    }
+
     /// Returns a [char] representing a single unicode point.
     ///
     /// If an event is processed, the return value is the `id` field from that
@@ -555,10 +580,9 @@ impl NcDirect {
     /// Provide a None `time` to block at length, a `time` of 0 for non-blocking
     /// operation, and otherwise a timespec to bound blocking.
     ///
-    /// *C style function: [ncdirect_getc()][crate::ncdirect_getc].*
-    //
+    /// *C style function: [ncdirect_get()][crate::ncdirect_get].*
     // CHECK returns 0 on a timeout.
-    pub fn getc(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
+    pub fn get(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
         let ntime;
         if let Some(time) = time {
             ntime = &time as *const _;
@@ -571,25 +595,36 @@ impl NcDirect {
         } else {
             ninput = null_mut();
         }
-        let c = unsafe {
-            core::char::from_u32_unchecked(crate::ncdirect_getc(self, ntime, null_mut(), ninput))
-        };
-        if c as u32 as i32 == NCRESULT_ERR {
-            return Err(NcError::new());
-        }
-        Ok(c)
+
+        let res = unsafe { crate::ncdirect_get(self, ntime, ninput) };
+        core::char::from_u32(res)
+            .ok_or_else(|| NcError::with_msg(res as i32, &format!["Nc.get(time: {:?})", time]))
     }
 
+    /// Reads input blocking until an event is processed or a signal is received.
     ///
-    /// *C style function: [ncdirect_getc_nblock()][crate::ncdirect_getc_nblock].*
-    pub fn getc_nblock(&mut self, input: &mut NcInput) -> char {
-        crate::ncdirect_getc_nblock(self, input)
-    }
-
+    /// Will optionally write the event details in `input`.
+    ///
+    /// In the case of a valid read, a [`char`] is returned.
     ///
     /// *C style function: [ncdirect_getc_blocking()][crate::ncdirect_getc_blocking].*
-    pub fn getc_blocking(&mut self, input: &mut NcInput) -> char {
-        crate::ncdirect_getc_blocking(self, input)
+    pub fn getc_blocking(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
+        let res = crate::ncdirect_getc_blocking(self, input);
+        core::char::from_u32(res as u32)
+            .ok_or_else(|| NcError::with_msg(res, "NcDirect.getc_blocking()"))
+    }
+
+    /// Reads input without blocking.
+    ///
+    /// In the case of a valid read, a [`char`] is returned.
+    ///
+    /// If no event is ready, returns 0.
+    ///
+    /// *C style function: [ncdirect_getc_nblock()][crate::ncdirect_getc_nblock].*
+    pub fn getc_nblock(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
+        let res = crate::ncdirect_getc_nblock(self, input);
+        core::char::from_u32(res as u32)
+            .ok_or_else(|| NcError::with_msg(res, "NcDirect.getc_nblock()"))
     }
 
     /// Get a file descriptor suitable for input event poll()ing.
@@ -631,10 +666,12 @@ impl NcDirect {
     /// be provided to the constructor.
     ///
     /// *C style function: [ncdirect_readline()][crate::ncdirect_readline].*
-    pub fn readline(&mut self, prompt: &str) -> NcResult<&str> {
+    //
+    // FIXME: memory leak still reported by valgrind
+    pub fn readline(&mut self, prompt: &str) -> NcResult<String> {
         let res = unsafe { crate::ncdirect_readline(self, cstring![prompt]) };
         if !res.is_null() {
-            return Ok(rstring![res]);
+            return Ok(rstring_free![res]);
         } else {
             Err(NcError::with_msg(
                 NCRESULT_ERR,
