@@ -18,7 +18,7 @@
 //!
 //! ### Example
 //!
-//! ```rust
+//! ```ignore
 //! use libnotcurses_sys::*;
 //!
 //! # #[cfg(not(miri))]
@@ -63,7 +63,7 @@
 //!
 //! ### Example
 //!
-//! ```rust
+//! ```ignore
 //! use core::ptr::{null, null_mut};
 //! use std::process::exit;
 //!
@@ -126,6 +126,7 @@ mod dimension;
 mod direct;
 mod error;
 mod fade;
+mod fd;
 mod file;
 mod input;
 mod macros;
@@ -148,6 +149,7 @@ pub use dimension::*;
 pub use direct::*;
 pub use error::*;
 pub use fade::*;
+pub use fd::*;
 pub use file::*;
 pub use macros::*;
 pub use metric::*;
@@ -160,3 +162,6 @@ pub use resizecb::*;
 pub use stats::*;
 pub use time::*;
 pub use visual::*;
+
+// re-export
+pub use libc;
