@@ -3,11 +3,11 @@
 use core::ptr::{null, null_mut};
 
 use crate::{
-    cstring, error, error_ref_mut, notcurses_init, rstring, Nc, NcAlign, NcBlitter, NcChannels,
-    NcDim, NcError, NcFile, NcInput, NcLogLevel, NcOptions, NcPlane, NcResult, NcScale, NcStats,
-    NcStyle, NcStyleMethods, NcTime, NCOPTION_NO_ALTERNATE_SCREEN, NCOPTION_SUPPRESS_BANNERS,
-    NCRESULT_ERR, NCSTYLE_BLINK, NCSTYLE_BOLD, NCSTYLE_ITALIC, NCSTYLE_NONE, NCSTYLE_STRUCK,
-    NCSTYLE_UNDERCURL, NCSTYLE_UNDERLINE,
+    cstring, error, error_ref_mut, notcurses_init, rstring, rstring_free, Nc, NcAlign, NcBlitter,
+    NcChannels, NcDim, NcError, NcFile, NcInput, NcLogLevel, NcOptions, NcPixelImpl, NcPlane,
+    NcResult, NcScale, NcStats, NcStyle, NcStyleMethods, NcTime, NCOPTION_NO_ALTERNATE_SCREEN,
+    NCOPTION_NO_CLEAR_BITMAPS, NCOPTION_PRESERVE_CURSOR, NCOPTION_SUPPRESS_BANNERS, NCSTYLE_BOLD,
+    NCSTYLE_ITALIC, NCSTYLE_NONE, NCSTYLE_STRUCK, NCSTYLE_UNDERCURL, NCSTYLE_UNDERLINE,
 };
 
 /// # `NcOptions` Constructors
@@ -68,8 +68,8 @@ impl NcOptions {
     ) -> Self {
         Self {
             termtype: null(),
-            renderfp: null_mut(),
             loglevel,
+            renderfp: null_mut(),
             margin_t: margin_t as i32,
             margin_r: margin_r as i32,
             margin_b: margin_b as i32,
@@ -80,12 +80,30 @@ impl NcOptions {
 }
 
 /// # `Nc` Constructors
+//
+// TODO: rethink constructors
 impl Nc {
     /// New notcurses context (without banners).
     pub fn new<'a>() -> NcResult<&'a mut Nc> {
         Self::with_flags(NCOPTION_SUPPRESS_BANNERS)
     }
 
+    /// New notcurses context in CLI mode.
+    ///
+    /// It has the following flags:
+    /// - [`NCOPTION_SUPPRESS_BANNERS`]
+    /// - [`NCOPTION_NO_ALTERNATE_SCREEN`]
+    /// - [`NCOPTION_NO_CLEAR_BITMAPS`]
+    /// - [`NCOPTION_PRESERVE_CURSOR`]
+    pub fn new_cli<'a>() -> NcResult<&'a mut Nc> {
+        Self::with_flags(
+            NCOPTION_SUPPRESS_BANNERS
+                | NCOPTION_NO_ALTERNATE_SCREEN
+                | NCOPTION_NO_CLEAR_BITMAPS
+                | NCOPTION_PRESERVE_CURSOR,
+        )
+    }
+
     /// New notcurses context, with banners.
     ///
     /// This is the default in the C library.
@@ -94,6 +112,8 @@ impl Nc {
     }
 
     /// New notcurses context, without an alternate screen (nor banners).
+    #[deprecated]
+    #[doc(hidden)]
     pub fn without_altscreen<'a>() -> NcResult<&'a mut Nc> {
         Self::with_flags(NCOPTION_NO_ALTERNATE_SCREEN | NCOPTION_SUPPRESS_BANNERS)
     }
@@ -106,7 +126,7 @@ impl Nc {
     /// New notcurses context, expects [NcOptions].
     pub fn with_options<'a>(options: NcOptions) -> NcResult<&'a mut Nc> {
         let res = unsafe { notcurses_init(&options, null_mut()) };
-        error_ref_mut![res, "Notcurses.with_options()"]
+        error_ref_mut![res, "Nc.with_options()"]
     }
 
     /// New notcurses context, expects [NcLogLevel] and flags.
@@ -134,9 +154,6 @@ impl Nc {
     /// as last rendered, returning the `EGC` (or None on error) and writing
     /// out the [`NcStyle`] and the [`NcChannels`].
     ///
-    // possible BUG? CHECK:
-    /// This `EGC` must be freed by the caller.
-    ///
     /// *C style function: [notcurses_at_yx()][crate::notcurses_at_yx].*
     pub fn at_yx(
         &mut self,
@@ -149,7 +166,7 @@ impl Nc {
         if egc.is_null() {
             return None;
         }
-        Some(rstring![egc].into())
+        Some(rstring_free![egc])
     }
 
     /// Returns the bottommost [`NcPlane`] on the standard pile,
@@ -251,22 +268,14 @@ impl Nc {
 
     /// Checks for pixel support.
     ///
-    /// Returns `false` for no support, or `true` if pixel output is supported.
-    ///
-    /// This function must successfully return before
-    /// [NCBLIT_PIXEL][crate::NCBLIT_PIXEL] is available.
-    ///
-    /// Must not be called concurrently with either input or rasterization.
+    /// Returns [`NcPixelImpl`] with a non-zero constant corresponding to some
+    /// pixel-blitting mechanism if bitmap support (via any mechanism) has been
+    /// detected, or else 0 (NCPIXEL_NONE).
     ///
     /// *C style function: [notcurses_check_pixel_support()][crate::notcurses_check-pixel_support].*
     #[allow(clippy::wildcard_in_or_patterns)]
-    pub fn check_pixel_support(&self) -> NcResult<bool> {
-        let res = unsafe { crate::notcurses_check_pixel_support(self) };
-        match res {
-            0 => Ok(false),
-            1 => Ok(true),
-            NCRESULT_ERR | _ => Err(NcError::with_msg(res, "Notcuses.check_pixel_support()")),
-        }
+    pub fn check_pixel_support(&self) -> NcPixelImpl {
+        unsafe { crate::notcurses_check_pixel_support(self) }
     }
 
     /// Disables the terminal's cursor, if supported.
@@ -288,6 +297,30 @@ impl Nc {
         error![unsafe { crate::notcurses_cursor_enable(self, y as i32, x as i32) }]
     }
 
+    /// Shifts to the alternate screen, if available.
+    ///
+    /// If already using the alternate screen, this returns Ok(()) immediately.
+    ///
+    /// If the alternate screen is not available, returns an Error immediately.
+    ///
+    /// Entering the alternate screen turns off scrolling for the standard plane.
+    ///
+    /// *C style function:
+    /// [notcurses_enter_alternate_screen()][crate::notcurses_enter_alternate_screen].*
+    pub fn enter_alternate_screen(&mut self) -> NcResult<()> {
+        error![unsafe { crate::notcurses_enter_alternate_screen(self) }]
+    }
+
+    /// Exits the alternate screen.
+    ///
+    /// Immediately returns Ok(()) if not currently using the alternate screen.
+    ///
+    /// *C style function:
+    /// [notcurses_leave_alternate_screen()][crate::notcurses_leave_alternate_screen].*
+    pub fn leave_alternate_screen(&mut self) -> NcResult<()> {
+        error![unsafe { crate::notcurses_leave_alternate_screen(self) }]
+    }
+
     /// Dumps notcurses state to the supplied `debugfp`.
     ///
     /// Output is freeform, and subject to change. It includes geometry of all
@@ -300,11 +333,25 @@ impl Nc {
         }
     }
 
+    /// Returns the name of the user under which we are running.
+    ///
+    /// *C style function: [notcurses_accountname()][crate::notcurses_accountname].*
+    pub fn accountname() -> String {
+        rstring_free![crate::notcurses_accountname()]
+    }
+
+    /// Returns the name of the local hostname.
+    ///
+    /// *C style function: [notcurses_hostname()][crate::notcurses_hostname].*
+    pub fn hostname() -> String {
+        rstring_free![crate::notcurses_hostname()]
+    }
+
     /// Returns the name of the detected terminal.
     ///
     /// *C style function: [notcurses_detected_terminal()][crate::notcurses_detected_terminal].*
     pub fn detected_terminal(&self) -> String {
-        rstring![crate::notcurses_detected_terminal(self)].to_string()
+        rstring_free![crate::notcurses_detected_terminal(self)]
     }
 
     /// Destroys all [`NcPlane`]s other than the stdplane.
@@ -316,6 +363,12 @@ impl Nc {
         }
     }
 
+    #[doc(hidden)]
+    #[deprecated = "use `get` method instead"]
+    pub fn getc(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
+        self.get(time, input)
+    }
+
     /// Returns a [char] representing a single unicode point.
     ///
     /// If an event is processed, the return value is the `id` field from that
@@ -324,8 +377,8 @@ impl Nc {
     /// Provide a None `time` to block at length, a `time` of 0 for non-blocking
     /// operation, and otherwise a timespec to bound blocking.
     ///
-    /// *C style function: [notcurses_getc()][crate::notcurses_getc].*
-    pub fn getc(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
+    /// *C style function: [notcurses_get()][crate::notcurses_get].*
+    pub fn get(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
         let ntime;
         if let Some(time) = time {
             ntime = &time as *const _;
@@ -339,46 +392,57 @@ impl Nc {
         } else {
             ninput = null_mut();
         }
-        let c = unsafe {
-            core::char::from_u32_unchecked(crate::notcurses_getc(self, ntime, null_mut(), ninput))
-        };
-        if c as u32 as i32 == NCRESULT_ERR {
-            return Err(NcError::new());
+
+        let res = unsafe { crate::notcurses_get(self, ntime, ninput) };
+        core::char::from_u32(res)
+            .ok_or_else(|| NcError::with_msg(res as i32, &format!["Nc.get(time: {:?})", time]))
+    }
+
+    /// Acquire up to 'vcount' [`NcInput`]s at the vector 'ni'.
+    ///
+    /// The number read will be returned, or 0 on timeout.
+    ///
+    /// *C style function: [notcurses_getvec()][crate::notcurses_getvec].*
+    pub fn getvec(
+        &mut self,
+        time: Option<NcTime>,
+        ni: &mut Vec<NcInput>,
+        vcount: u32,
+    ) -> NcResult<u32> {
+        let ntime;
+        if let Some(time) = time {
+            ntime = &time as *const _;
+        } else {
+            ntime = null();
         }
-        Ok(c)
+        let nivec = ni.as_mut_ptr() as *mut NcInput;
+
+        let res = unsafe { crate::notcurses_getvec(self, ntime, nivec, vcount as i32) };
+        error![res, "", res as u32]
     }
 
-    /// If no event is ready, returns 0.
+    /// Reads input blocking until an event is processed or a signal is received.
     ///
-    /// *C style function: [notcurses_getc_nblock()][crate::notcurses_getc_nblock].*
-    pub fn getc_nblock(&mut self, input: &mut NcInput) -> char {
-        crate::notcurses_getc_nblock(self, input)
-    }
-
-    /// Blocks until a codepoint or special key is read,
-    /// or until interrupted by a signal.
+    /// Will optionally write the event details in `input`.
     ///
-    /// In the case of a valid read, a 32-bit Unicode codepoint is returned.
-    ///
-    /// Optionally writes the event details in `input`.
+    /// In the case of a valid read, a [`char`] is returned.
     ///
     /// *C style function: [notcurses_getc_blocking()][crate::notcurses_getc_blocking].*
     pub fn getc_blocking(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
-        let input_txt;
-        if cfg!(debug_assertions) {
-            input_txt = format!("{:?}", input);
-        } else {
-            input_txt = String::from("");
-        }
-
         let res = crate::notcurses_getc_blocking(self, input);
+        core::char::from_u32(res as u32).ok_or_else(|| NcError::with_msg(res, "Nc.getc_blocking()"))
+    }
 
-        // An invalid read is indicated with -1
-        if res as u32 as i32 != -1 {
-            Ok(res)
-        } else {
-            error![-1, &format!("Nc.getc_blocking({:?})", input_txt), res]
-        }
+    /// Reads input without blocking.
+    ///
+    /// In the case of a valid read, a [`char`] is returned.
+    ///
+    /// If no event is ready, returns 0.
+    ///
+    /// *C style function: [notcurses_getc_nblock()][crate::notcurses_getc_nblock].*
+    pub fn getc_nblock(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
+        let res = crate::notcurses_getc_nblock(self, input);
+        core::char::from_u32(res as u32).ok_or_else(|| NcError::with_msg(res, "Nc.getc_nblock()"))
     }
 
     /// Gets a file descriptor suitable for input event poll()ing.
@@ -432,7 +496,7 @@ impl Nc {
     /// spaces.
     ///
     /// The supported styles are: `italic`, `underline`, `undercurl`,
-    /// `struck`, `bold`, `blink` and `none`.
+    /// `struck`, `bold`, and `none`.
     ///
     /// If a style is are not recognized returns an error.
     ///
@@ -448,7 +512,6 @@ impl Nc {
                 "undercurl" => style.add(NCSTYLE_UNDERCURL),
                 "struck" => style.add(NCSTYLE_STRUCK),
                 "bold" => style.add(NCSTYLE_BOLD),
-                "blink" => style.add(NCSTYLE_BLINK),
                 "none" => (),
                 _ => {
                     errstr.push_str(s);
@@ -561,9 +624,9 @@ impl Nc {
     /// *C style function: [notcurses_render_to_buffer()][crate::notcurses_render_to_buffer].*
     //
     // CHECK that this works.
+    #[deprecated]
     pub fn render_to_buffer(&mut self, buffer: &mut Vec<u8>) -> NcResult<()> {
-        #[allow(unused_mut)] // CHECK whether it actually works without the mut
-        let mut len = buffer.len() as u32;
+        let len = buffer.len() as u32;
 
         // https://github.com/dankamongmen/notcurses/issues/1339
         #[cfg(any(target_arch = "x86_64", target_arch = "i686", target_arch = "x86"))]
@@ -580,6 +643,7 @@ impl Nc {
     /// nothing will be written.
     ///
     /// *C style function: [notcurses_render_to_file()][crate::notcurses_render_to_file].*
+    #[deprecated]
     pub fn render_to_file(&mut self, fp: &mut NcFile) -> NcResult<()> {
         error![unsafe { crate::notcurses_render_to_file(self, fp.as_nc_ptr()) }]
     }
@@ -627,7 +691,7 @@ impl Nc {
     //     crate::notcurses_stddim_yx(self, y, x)
     // }
 
-    // /// [stdplane_const()][Notcurses#method.stdplane_const], plus free
+    // /// [stdplane_const()][Nc#method.stdplane_const], plus free
     // /// bonus dimensions written to non-NULL y/x!
     // ///
     // /// *C style function: [notcurses_stddim_yx()][crate::notcurses_stddim_yx].*
@@ -693,9 +757,9 @@ impl Nc {
                 NCSTYLE_UNDERCURL => "undercurl",
                 NCSTYLE_STRUCK => "struck",
                 NCSTYLE_BOLD => "bold",
-                NCSTYLE_BLINK => "blink",
+                #[allow(unreachable_patterns)] // FIXME
                 NCSTYLE_NONE => "none",
-                _ => "",
+                _ => "none",
             });
             string.push(' ');
         }
