@@ -32,7 +32,8 @@
 // fmt notcurses_debug
 //~f   notcurses_detected_terminal
 // fmt notcurses_drop_planes
-// fm  notcurses_getc
+// fm  notcurses_get
+// fm  notcurses_getvec
 // fmt notcurses_init
 // fm  notcurses_inputready_fd
 // fm  notcurses_lex_blitter
@@ -88,13 +89,20 @@ pub type Nc = crate::bindings::ffi::notcurses;
 #[doc(hidden)]
 pub type Notcurses = Nc;
 
-/// Options struct for [`Notcurses`]
+/// Options struct for [`Nc`]
 pub type NcOptions = crate::bindings::ffi::notcurses_options;
 
 #[deprecated]
 #[doc(hidden)]
 pub type NotcursesOptions = NcOptions;
 
+/// Input may be freely dropped.
+///
+/// This ought be provided when the program does not intend to handle input.
+/// Otherwise, input can accumulate in internal buffers, eventually preventing
+/// Notcurses from processing terminal messages.
+pub const NCOPTION_DRAIN_INPUT: u64 = crate::bindings::ffi::NCOPTION_DRAIN_INPUT as u64;
+
 /// Do not call setlocale()
 ///
 /// notcurses_init() will call setlocale() to inspect the current locale. If
@@ -196,17 +204,18 @@ pub const NCLOGLEVEL_VERBOSE: NcLogLevel = crate::bindings::ffi::ncloglevel_e_NC
 /// you probably don't want what's happening to happen
 pub const NCLOGLEVEL_WARNING: NcLogLevel = crate::bindings::ffi::ncloglevel_e_NCLOGLEVEL_WARNING;
 
-// NcAlign -- ------------------------------------------------------------------
+// NcAlign ---------------------------------------------------------------------
 
 /// Alignment within a plane or terminal.
 /// Left/right-justified, or centered.
 ///
 /// ## Defined constants
-///
-/// - [NCALIGN_UNALIGNED]
-/// - [NCALIGN_LEFT]
-/// - [NCALIGN_CENTER]
-/// - [NCALIGN_RIGHT]
+/// - [`NCALIGN_UNALIGNED`]
+/// - [`NCALIGN_LEFT`]
+/// - [`NCALIGN_RIGHT`]
+/// - [`NCALIGN_TOP`]
+/// - [`NCALIGN_BOTTOM`]
+/// - [`NCALIGN_CENTER`]
 pub type NcAlign = crate::bindings::ffi::ncalign_e;
 
 /// Left alignment within an [`NcPlane`][crate::NcPlane] or terminal.
@@ -215,8 +224,48 @@ pub const NCALIGN_LEFT: NcAlign = crate::bindings::ffi::ncalign_e_NCALIGN_LEFT;
 /// Right alignment within an [`NcPlane`][crate::NcPlane] or terminal.
 pub const NCALIGN_RIGHT: NcAlign = crate::bindings::ffi::ncalign_e_NCALIGN_RIGHT;
 
+/// Top alignment within an [`NcPlane`][crate::NcPlane] or terminal.
+pub const NCALIGN_TOP: NcAlign = NCALIGN_LEFT;
+
+/// Bottom alignment within an [`NcPlane`][crate::NcPlane] or terminal.
+pub const NCALIGN_BOTTOM: NcAlign = NCALIGN_RIGHT;
+
 /// Center alignment within an [`NcPlane`][crate::NcPlane] or terminal.
 pub const NCALIGN_CENTER: NcAlign = crate::bindings::ffi::ncalign_e_NCALIGN_CENTER;
 
-/// Do not align an [`NcPlane`][crate::NcPlane] or terminal.
+/// Nothing unaligned should appear.
 pub const NCALIGN_UNALIGNED: NcAlign = crate::bindings::ffi::ncalign_e_NCALIGN_UNALIGNED;
+
+// NcPixelImpl -----------------------------------------------------------------
+
+/// Pixel blitting implementations. (Informative only).
+///
+/// Returned by [`check_pixel_support`][Nc#method.check_pixel_support].
+///
+/// ## Defined constants
+/// - [`NCPIXEL_NONE`]
+/// - [`NCPIXEL_SIXEL`]
+/// - [`NCPIXEL_LINUXFB`]
+/// - [`NCPIXEL_ITERM2`]
+/// - [`NCPIXEL_KITTY_STATIC`]
+/// - [`NCPIXEL_KITTY_ANIMATED`]
+/// - [`NCPIXEL_KITTY_SELFREF`]
+pub type NcPixelImpl = crate::bindings::ffi::ncpixelimpl_e;
+
+/// No pixel support (for [`NcPixelImpl`]).
+pub const NCPIXEL_NONE: NcPixelImpl = crate::bindings::ffi::ncpixelimpl_e_NCPIXEL_NONE;
+/// Sixel (for [`NcPixelImpl`]).
+pub const NCPIXEL_SIXEL: NcPixelImpl = crate::bindings::ffi::ncpixelimpl_e_NCPIXEL_SIXEL;
+/// Linux framebuffer (for [`NcPixelImpl`]).
+pub const NCPIXEL_LINUXFB: NcPixelImpl = crate::bindings::ffi::ncpixelimpl_e_NCPIXEL_LINUXFB;
+/// iTerm2 (for [`NcPixelImpl`]).
+pub const NCPIXEL_ITERM2: NcPixelImpl = crate::bindings::ffi::ncpixelimpl_e_NCPIXEL_ITERM2;
+/// Kitty prior to C=1 and animation (for [`NcPixelImpl`]).
+pub const NCPIXEL_KITTY_STATIC: NcPixelImpl =
+    crate::bindings::ffi::ncpixelimpl_e_NCPIXEL_KITTY_STATIC;
+/// Kitty with animation but not reflexive composition (for [`NcPixelImpl`]).
+pub const NCPIXEL_KITTY_ANIMATED: NcPixelImpl =
+    crate::bindings::ffi::ncpixelimpl_e_NCPIXEL_KITTY_ANIMATED;
+/// Kitty with reflexive composition (for [`NcPixelImpl`]).
+pub const NCPIXEL_KITTY_SELFREF: NcPixelImpl =
+    crate::bindings::ffi::ncpixelimpl_e_NCPIXEL_KITTY_SELFREF;
