@@ -1,21 +1,22 @@
 //! `NcPlane`
 
-// functions already exported by bindgen : 117
+// functions already exported by bindgen : 103
 // -------------------------------------------
-// (X) wont: 10
-// (D) depr:  4
-// (#) test: 13
-// (W) wrap: 83
+// (#) test: 9
+// (W) wrap: 95
 // -------------------------------------------
 //W  ncpile_bottom
 //W# ncpile_create
 //W  ncpile_rasterize
 //W  ncpile_render
+//W  ncpile_render_to_buffer,
+//W  ncpile_render_to_file,
 //W  ncpile_top
 //W  ncplane_above
 //W  ncplane_abs_x
 //W  ncplane_abs_y
 //W  ncplane_abs_yx
+//W  ncplane_as_rgba
 //W  ncplane_at_cursor
 //W  ncplane_at_cursor_cell
 //W  ncplane_at_yx
@@ -24,24 +25,22 @@
 //W  ncplane_below
 //W  ncplane_box
 //W  ncplane_center_abs
-//W# ncplane_channels
 //W  ncplane_contents
 //W  ncplane_create
+//W  ncplane_cursor_move_rel
 //W# ncplane_cursor_move_yx
 //W# ncplane_cursor_yx
 //W  ncplane_destroy
 //W# ncplane_dim_yx
 //W  ncplane_dup
 //W# ncplane_erase
-//   ncplane_erase_region
+//W  ncplane_erase_region
 //W  ncplane_fadein
 //W  ncplane_fadein_iteration
 //W  ncplane_fadeout
 //W  ncplane_fadeout_iteration
 //W  ncplane_format
-//W  ncplane_gradient
 //W  ncplane_greyscale
-//   ncplane_halign
 //W  ncplane_highgradient
 //W  ncplane_highgradient_sized
 //   ncplane_hline_interp
@@ -51,9 +50,10 @@
 //W  ncplane_move_above
 //W  ncplane_move_below
 //W  ncplane_move_bottom
+//W  ncplane_move_family_above
+//W  ncplane_move_family_below
 //W  ncplane_move_top
 //W  ncplane_move_yx
-// X ncplane_new                       // deprecated
 //W# ncplane_notcurses
 //W# ncplane_notcurses_const
 //W  ncplane_off_styles
@@ -63,74 +63,57 @@
 //W  ncplane_pixelgeom
 //   ncplane_polyfill_yx
 //W  ncplane_pulse
-//   ncplane_putchar_stained
-//   ncplane_putc_yx
-// X ncplane_putegc_stained            // unneeded
-// X ncplane_putegc_yx                 // unneeded
-//   ncplane_putnstr_aligned
-//   ncplane_putnstr_yx
+//W  ncplane_putc_yx
+//W  ncplane_putnstr_aligned
+//W  ncplane_putnstr_yx
 //W  ncplane_putstr_aligned
 //W  ncplane_putstr_stained
 //W  ncplane_putstr_yx
-//   ncplane_puttext
-// X ncplane_putwegc_stained           // unneeded
-// X ncplane_putwstr_stained           // unneeded
+//W  ncplane_puttext
 //   ncplane_qrcode
 //W  ncplane_reparent
 //W  ncplane_reparent_family
 //W# ncplane_resize
-//W  ncplane_resizecb
-//W  ncplane_resize_marginalize
+//W  ncplane_resize_marginalized
 //W  ncplane_resize_maximize
 //W  ncplane_resize_realign
-//W  ncplane_rgba
+//W  ncplane_resizecb
 //W  ncplane_rotate_ccw
 //W  ncplane_rotate_cw
 //W  ncplane_scrolling_p
+//W  ncplane_scrollup,
+//W  ncplane_scrollup_child,
 //W  ncplane_set_base
 //W  ncplane_set_base_cell
-//W# ncplane_set_bchannel
 //W  ncplane_set_bg_alpha
 //W  ncplane_set_bg_default
 //W  ncplane_set_bg_palindex
 //W  ncplane_set_bg_rgb
 //W  ncplane_set_bg_rgb8
-// X ncplane_set_bg_rgb8_clipped       // unneeded
-//W# ncplane_set_channels
-//W# ncplane_set_fchannel
 //W  ncplane_set_fg_alpha
 //W  ncplane_set_fg_default
 //W  ncplane_set_fg_palindex
 //W  ncplane_set_fg_rgb
 //W  ncplane_set_fg_rgb8
-// X ncplane_set_fg_rgb8_clipped       // unneeded
 //W  ncplane_set_resizecb
 //W  ncplane_set_scrolling
 //W  ncplane_set_styles
 //   ncplane_set_userptr
 //W  ncplane_stain
 //W  ncplane_styles
-// X ncplane_styles_off                // deprecated
-// X ncplane_styles_on                 // deprecated
-// X ncplane_styles_set                // deprecated
 //W  ncplane_translate
 //W  ncplane_translate_abs
 //   ncplane_userptr
-//   ncplane_valign
 //   ncplane_vline_interp
-// X ncplane_vprintf_aligned
-// X ncplane_vprintf_stained
-// X ncplane_vprintf_yx
 //W  ncplane_x
 //W  ncplane_y
 //W  ncplane_yx
-//
-// functions manually reimplemented: 40
+
+// functions manually reimplemented: 43
 // ------------------------------------------
-// (X) wont:  9
-// (+) done: 33 / 0
-// (W) wrap: 25
-// (#) test:  5
+// (+) done: 43
+// (W) wrap: 39
+// (#) test:  9
 // ------------------------------------------
 //W+ ncplane_bchannel
 //W+ ncplane_bg_alpha
@@ -138,6 +121,7 @@
 //W+ ncplane_bg_rgb
 //W+ ncplane_bg_rgb8
 //W+ ncplane_box_sized
+//W# ncplane_channels
 //W# ncplane_dim_x
 //W# ncplane_dim_y
 //W+ ncplane_double_box
@@ -147,32 +131,56 @@
 //W# ncplane_fg_default_p
 //W+ ncplane_fg_rgb
 //W+ ncplane_fg_rgb8
+//W+ ncplane_gradient
 //W+ ncplane_gradient_sized
 //W+ ncplane_halign
 // + ncplane_hline
+//W+ ncplane_moverel
+//W+ ncplane_move_family_bottom
+//W+ ncplane_move_family_top
 //W+ ncplane_perimeter
 //W+ ncplane_perimeter_double
 //W+ ncplane_perimeter_rounded
-// + ncplane_putc
-// + ncplane_putchar
-// + ncplane_putchar_yx
-// X ncplane_putegc                    // unneeded
-// + ncplane_putnstr
+//W+ ncplane_putc
+//W+ ncplane_putchar
+//W+ ncplane_putchar_stained
+//W+ ncplane_putchar_yx
+//W+ ncplane_putegc
+//W+ ncplane_putegc_stained
+//W+ ncplane_putegc_yx
+//W+ ncplane_putnstr
 //W+ ncplane_putstr
-// X ncplane_putwc                     // unneeded
-// X ncplane_putwc_stained             // unneeded
-// X ncplane_putwc_yx                  // unneeded
-// X ncplane_putwegc                   // unneeded
-// X ncplane_putwegc_yx                // unneeded
-// X ncplane_putwstr                   // unneeded
-// X ncplane_putwstr_aligned           // unneeded
-// X ncplane_putwstr_yx                // unneeded
 //W# ncplane_resize_simple
 // + ncplane_rounded_box
 // + ncplane_rounded_box_sized
-//W+ ncplane_halign
+//W# ncplane_set_bchannel
+//W# ncplane_set_channels
+//W# ncplane_set_fchannel
+//W+ ncplane_valign
 // + ncplane_vline
-// X ncplane_vprintf
+
+// wont implement:
+// ----------------
+//   ncplane_putegc_stained
+//   ncplane_putegc_yx
+//   ncplane_putwegc_stained
+//   ncplane_putwstr_stained
+//   ncplane_set_bg_rgb8_clipped
+//   ncplane_set_fg_rgb8_clipped
+//   ncplane_vprintf_aligned
+//   ncplane_vprintf_stained
+//   ncplane_vprintf_yx
+//
+//   ncplane_putegc
+//   ncplane_putwc
+//   ncplane_putwc_stained
+//   ncplane_putwc_yx
+//   ncplane_putwegc
+//   ncplane_putwegc_yx
+//   ncplane_putwstr
+//   ncplane_putwstr_aligned
+//   ncplane_putwstr_yx
+//   ncplane_vprintf
 
 #[cfg(test)]
 mod test;
@@ -251,7 +259,7 @@ pub use reimplemented::*;
 /// the various *output calls* only draw to the virtual `NcPlane`s. Most of the
 /// notcurses `statistics` are updated as a result of a render, and screen
 /// geometry is refreshed (similarly to
-/// [`Notcurses.refresh`][crate::Notcurses#method.refresh]) following the render.
+/// [`Nc.refresh`][crate::Nc#method.refresh]) following the render.
 ///
 /// # Methods & Associated Functions
 ///
@@ -264,7 +272,7 @@ pub use reimplemented::*;
 /// - [`NcStyle` & `NcPaletteIndex`](#ncplane-methods-ncstylemask--paletteindex)
 /// - [`NcCell` & strings](#ncplane-methods-nccell--strings)
 /// - [cursor](#ncplane-methods-cursor)
-/// - [`NcPlane` & `Notcurses`](#ncplane-methods-ncplane--notcurses)
+/// - [`NcPlane` & `Nc`](#ncplane-methods-ncplane--nc)
 /// - [boxes & perimeters](#ncplane-methods-boxes--perimeters)
 /// - [Size, position & alignment](#ncplane-methods-size-position--alignment)
 /// - [fading, gradients & greyscale](#ncplane-methods-fading-gradients--greyscale)
@@ -289,13 +297,3 @@ pub const NCPLANE_OPTION_VERALIGNED: u64 = crate::bindings::ffi::NCPLANE_OPTION_
 /// used. This flag is exclusive with both of the alignment flags.
 pub const NCPLANE_OPTION_MARGINALIZED: u64 =
     crate::bindings::ffi::NCPLANE_OPTION_MARGINALIZED as u64;
-
-/// I/O wrapper to dump file descriptor to [`NcPlane`]
-///
-/// `type in C: ncfdplane (struct)`
-pub type NcFdPlane = crate::bindings::ffi::ncfdplane;
-
-/// Options struct for [`NcFdPlane`]
-///
-/// `type in C: ncplane_options (struct)`
-pub type NcFdPlaneOptions = crate::bindings::ffi::ncfdplane_options;
