@@ -3,8 +3,9 @@
 use core::ptr::null_mut;
 
 use crate::{
-    cstring, nccell_release, NcAlign, NcAlphaBits, NcBoxMask, NcCell, NcChannel, NcChannels,
-    NcComponent, NcDim, NcIntResult, NcOffset, NcPlane, NcRgb, NcStyle, NCRESULT_ERR, NCRESULT_OK,
+    cstring, ffi::size_t, nccell_release, NcAlign, NcAlphaBits, NcBoxMask, NcCell, NcChannel,
+    NcChannels, NcComponent, NcDim, NcIntResult, NcOffset, NcPlane, NcRgb, NcStyle, NCRESULT_ERR,
+    NCRESULT_OK,
 };
 
 // Alpha -----------------------------------------------------------------------
@@ -193,7 +194,15 @@ pub fn ncplane_set_bg_not_default(plane: &NcPlane) -> NcChannels {
 
 // put & print -----------------------------------------------------------------
 
-/// Calls [ncplane_putc_yx()][crate::ncplane_putc_yx] for the current cursor location.
+/// Replaces the [`NcCell`] at the current location with the provided `cell`,
+/// advancing the cursor by its width (but not past the end of the plane).
+///
+/// The new `cell` must already be associated with this `NcPlane`.
+///
+/// On success, returns the number of columns the cursor was advanced.
+///
+/// If the glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
 ///
 /// *Method: NcPlane.[putc()][NcPlane#method.putc].*
 #[inline]
@@ -201,37 +210,161 @@ pub fn ncplane_putc(plane: &mut NcPlane, cell: &NcCell) -> NcIntResult {
     unsafe { crate::ncplane_putc_yx(plane, -1, -1, cell) }
 }
 
-/// Calls ncplane_putchar_yx() at the current cursor location.
+/// Replaces the [`NcCell`] at the current location with the provided [`char`],
+/// using the current style.
+///
+/// On success, returns the number of columns the cursor was advanced.
+///
+/// If the glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
 ///
 /// *Method: NcPlane.[putchar()][NcPlane#method.putchar].*
 #[inline]
 pub fn ncplane_putchar(plane: &mut NcPlane, ch: char) -> NcIntResult {
     unsafe {
-        let cell = NcCell::from_char(ch, plane);
-        crate::ncplane_putc_yx(plane, -1, -1, &cell)
+        let cell = NcCell::from_char(plane, ch);
+        if cell.is_err() {
+            return NCRESULT_ERR;
+        }
+        crate::ncplane_putc_yx(plane, -1, -1, &cell.unwrap())
     }
 }
 
-/// Replaces the [NcCell] at the specified coordinates with the provided char.
-/// Advances the cursor by 1.
+/// Replaces the [`NcCell`] at the specified coordinates with the provided
+/// [`char`], using the current style.
+///
+/// On success, returns the number of columns the cursor was advanced.
+///
+/// If the glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
 ///
 /// *Method: NcPlane.[putchar_yx()][NcPlane#method.putchar_yx].*
 #[inline]
 pub fn ncplane_putchar_yx(plane: &mut NcPlane, y: NcDim, x: NcDim, ch: char) -> NcIntResult {
     unsafe {
-        let cell = NcCell::from_char(ch, plane);
-        crate::ncplane_putc_yx(plane, y as i32, x as i32, &cell)
+        let cell = NcCell::from_char(plane, ch);
+        if cell.is_err() {
+            return NCRESULT_ERR;
+        }
+        crate::ncplane_putc_yx(plane, y as i32, x as i32, &cell.unwrap())
     }
 }
 
-/// Writes a series of `EGC`s to the current location, using the current style.
+/// Replaces the [`NcCell`] at the current location with the provided
+/// [`char`], while retaining the previous style.
 ///
-/// Advances the cursor by some positive number of columns
-/// (though not beyond the end of the plane),
-/// and this number is returned on success.
+/// On success, returns the number of columns the cursor was advanced.
 ///
-/// On error, a non-positive number is returned, indicating
-/// the number of columns which were written before the error.
+/// If the glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
+///
+/// *Method: NcPlane.[putchar_stained()][NcPlane#method.putchar_stained].*
+#[inline]
+pub fn ncplane_putchar_stained(plane: &mut NcPlane, ch: char) -> NcIntResult {
+    unsafe { crate::ncplane_putstr_stained(plane, cstring![ch.to_string()]) }
+}
+
+/// Replaces the [`NcCell`] at the current location with the provided `egc`,
+/// using the current style.
+///
+/// Advances the cursor by the width of the cluster (but not past the end of
+/// the the plane), and this number is returned on success.
+///
+/// The number of bytes converted from the `egc` can be optionally written to
+/// `sbytes`.
+///
+/// If the glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
+///
+/// *Method: NcPlane.[putegc()][NcPlane#method.putegc].*
+//
+// MAYBE accept sbytes as Option<&mut usize>
+#[inline]
+pub fn ncplane_putegc(plane: &mut NcPlane, egc: &str, sbytes: Option<&mut i32>) -> NcIntResult {
+    let sbytes_ptr;
+    if let Some(sb) = sbytes {
+        sbytes_ptr = sb as *mut _;
+    } else {
+        sbytes_ptr = null_mut();
+    }
+
+    unsafe { crate::ffi::ncplane_putegc_yx(plane, -1, -1, cstring![egc], sbytes_ptr) }
+}
+
+/// Replaces the [`NcCell`] at the specified coordinates with the provided `egc`,
+/// using the current style.
+///
+/// Advances the cursor by the width of the cluster (but not past the end of
+/// the the plane), and this number is returned on success.
+///
+/// The number of bytes converted from the `egc` can be optionally written to
+/// `sbytes`.
+///
+/// If the glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
+///
+/// *Method: NcPlane.[putegc_yx()][NcPlane#method.putegc_yx].*
+//
+// MAYBE accept sbytes as Option<&mut usize>
+#[inline]
+pub fn ncplane_putegc_yx(
+    plane: &mut NcPlane,
+    y: NcDim,
+    x: NcDim,
+    egc: &str,
+    sbytes: Option<&mut i32>,
+) -> NcIntResult {
+    let sbytes_ptr;
+    if let Some(sb) = sbytes {
+        sbytes_ptr = sb as *mut _;
+    } else {
+        sbytes_ptr = null_mut();
+    }
+
+    unsafe { crate::ffi::ncplane_putegc_yx(plane, y as i32, x as i32, cstring![egc], sbytes_ptr) }
+}
+
+/// Replaces the [`NcCell`] at the current location with the provided `egc`,
+/// while retaining the previous style.
+///
+/// Advances the cursor by the width of the cluster (but not past the end of
+/// the the plane), and this number is returned on success.
+///
+/// The number of bytes converted from the `egc` can be optionally written to
+/// `sbytes`.
+///
+/// If the glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
+///
+/// *Method: NcPlane.[putegc_stained()][NcPlane#method.putegc_stained].*
+//
+// MAYBE accept sbytes as Option<&mut usize>
+#[inline]
+pub fn ncplane_putegc_stained(
+    plane: &mut NcPlane,
+    egc: &str,
+    sbytes: Option<&mut i32>,
+) -> NcIntResult {
+    let sbytes_ptr;
+    if let Some(sb) = sbytes {
+        sbytes_ptr = sb as *mut _;
+    } else {
+        sbytes_ptr = null_mut();
+    }
+
+    unsafe { crate::ffi::ncplane_putegc_stained(plane, cstring![egc], sbytes_ptr) }
+}
+
+/// Writes a string to the current location, using the current style.
+///
+/// Advances the cursor by some positive number of columns (though not beyond
+/// the end of the plane), and this number is returned on success.
+///
+/// On error, a non-positive number is returned, indicating the number of
+/// columns which were written before the error.
+///
+/// If a glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
 ///
 /// *Method: NcPlane.[putstr()][NcPlane#method.putstr].*
 #[inline]
@@ -239,12 +372,22 @@ pub fn ncplane_putstr(plane: &mut NcPlane, string: &str) -> NcIntResult {
     unsafe { crate::ncplane_putstr_yx(plane, -1, -1, cstring![string]) }
 }
 
+/// Writes a string to the current location, using the current style,
+/// and no more than `num_bytes` bytes will be written.
 ///
+/// Advances the cursor by some positive number of columns (though not beyond
+/// the end of the plane); this number is returned on success.
+///
+/// On error, a non-positive number is returned, indicating the number of
+/// columns which were written before the error.
+///
+/// If a glyph can not fit in the current line, it is an error, unless
+/// scrolling is enabled.
 ///
 /// *Method: NcPlane.[putnstr()][NcPlane#method.putnstr].*
 #[inline]
-pub fn ncplane_putnstr(plane: &mut NcPlane, size: u32, gclustarr: &[u8]) -> NcIntResult {
-    unsafe { crate::ncplane_putnstr_yx(plane, -1, -1, size.into(), cstring![gclustarr]) }
+pub fn ncplane_putnstr(plane: &mut NcPlane, num_bytes: usize, string: &str) -> NcIntResult {
+    unsafe { crate::ncplane_putnstr_yx(plane, -1, -1, num_bytes as size_t, cstring![string]) }
 }
 
 // movement, size & alignment --------------------------------------------------
@@ -256,15 +399,48 @@ pub fn ncplane_putnstr(plane: &mut NcPlane, size: u32, gclustarr: &[u8]) -> NcIn
 ///
 /// It is an error to attempt to move the standard plane.
 ///
-/// *C style function: [ncplane_moverel()][crate::ncplane_moverel].*
+/// *Method: NcPlane.[move_rel()][NcPlane#method.move_rel].*
+#[inline]
 pub fn ncplane_moverel(plane: &mut NcPlane, rows: NcOffset, cols: NcOffset) -> NcIntResult {
     let (mut orig_y, mut orig_x) = (0, 0);
     unsafe {
-        crate::ncplane_dim_yx(plane, &mut orig_y, &mut orig_x);
+        crate::ncplane_yx(plane, &mut orig_y, &mut orig_x);
         crate::ncplane_move_yx(plane, orig_y + rows, orig_x + cols)
     }
 }
 
+/// Splices this plane and its bound planes out of the z-buffer,
+/// and reinserts them at the top.
+///
+/// Relative order will be maintained between the reinserted planes.
+///
+/// For a plane E bound to C, with z-ordering A B C D E, moving the C family
+/// to the top results in C E A B D.
+///
+/// *Method: NcPlane.[move_family_top()][NcPlane#method.move_family_top].*
+#[inline]
+pub fn ncplane_move_family_top(plane: &mut NcPlane) {
+    unsafe {
+        crate::ncplane_move_family_below(plane, null_mut());
+    }
+}
+
+/// Splices this plane and its bound planes out of the z-buffer,
+/// and reinserts them at the bottom.
+///
+/// Relative order will be maintained between the reinserted planes.
+///
+/// For a plane E bound to C, with z-ordering A B C D E, moving the C family
+/// to the bottom results in A B D C E.
+///
+/// *Method: NcPlane.[move_family_bottom()][NcPlane#method.move_family_bottom].*
+#[inline]
+pub fn ncplane_move_family_bottom(plane: &mut NcPlane) {
+    unsafe {
+        crate::ncplane_move_family_above(plane, null_mut());
+    }
+}
+
 /// Gets the columns of the [NcPlane].
 ///
 /// *Method: NcPlane.[dim_x()][NcPlane#method.dim_x].*
@@ -295,21 +471,21 @@ pub fn ncplane_dim_y(plane: &NcPlane) -> NcDim {
 ///
 /// *Method: NcPlane.[resize_simple()][NcPlane#method.resize_simple].*
 #[inline]
-pub fn ncplane_resize_simple(plane: &mut NcPlane, y_len: NcDim, x_len: NcDim) -> NcIntResult {
+pub fn ncplane_resize_simple(plane: &mut NcPlane, len_y: NcDim, len_x: NcDim) -> NcIntResult {
     let (mut old_y, mut old_x) = (0, 0);
     unsafe {
         crate::ncplane_dim_yx(plane, &mut old_y, &mut old_x);
     }
     let keep_len_y = {
-        if old_y > y_len as i32 {
-            y_len as i32
+        if old_y > len_y as i32 {
+            len_y as i32
         } else {
             old_y
         }
     };
     let keep_len_x = {
-        if old_x > x_len as i32 {
-            x_len as i32
+        if old_x > len_x as i32 {
+            len_x as i32
         } else {
             old_x
         }
@@ -323,25 +499,25 @@ pub fn ncplane_resize_simple(plane: &mut NcPlane, y_len: NcDim, x_len: NcDim) ->
             keep_len_x,
             0,
             0,
-            y_len as i32,
-            x_len as i32,
+            len_y as i32,
+            len_x as i32,
         )
     }
 }
 
-/// Returns the column at which `cols` columns ought start in order to be aligned
-/// according to `align` within this NcPlane.
+/// Returns the column at which `numcols` columns ought start in order to be
+/// aligned according to `align` within the `plane`.
 ///
 /// Returns `-`[`NCRESULT_MAX`][crate::NCRESULT_MAX] if
 /// [NCALIGN_UNALIGNED][crate::NCALIGN_UNALIGNED] or invalid [NcAlign].
 ///
 /// *Method: NcPlane.[halign()][NcPlane#method.halign].*
 #[inline]
-pub fn ncplane_halign(plane: &NcPlane, align: NcAlign, cols: NcDim) -> NcIntResult {
-    crate::notcurses_align(ncplane_dim_x(plane), align, cols)
+pub fn ncplane_halign(plane: &NcPlane, align: NcAlign, numcols: NcDim) -> NcIntResult {
+    crate::notcurses_align(ncplane_dim_x(plane), align, numcols)
 }
 
-/// Returns the row at which `rows` rows ought start in order to be aligned
+/// Returns the row at which `numrows` rows ought start in order to be aligned
 /// according to `align` within this NcPlane.
 ///
 /// Returns `-`[`NCRESULT_MAX`][crate::NCRESULT_MAX] if
@@ -349,8 +525,8 @@ pub fn ncplane_halign(plane: &NcPlane, align: NcAlign, cols: NcDim) -> NcIntResu
 ///
 /// *Method: NcPlane.[valign()][NcPlane#method.valign].*
 #[inline]
-pub fn ncplane_valign(plane: &NcPlane, align: NcAlign, rows: NcDim) -> NcIntResult {
-    crate::notcurses_align(ncplane_dim_y(plane), align, rows)
+pub fn ncplane_valign(plane: &NcPlane, align: NcAlign, numrows: NcDim) -> NcIntResult {
+    crate::notcurses_align(ncplane_dim_y(plane), align, numrows)
 }
 
 // line ------------------------------------------------------------------------
@@ -359,7 +535,7 @@ pub fn ncplane_valign(plane: &NcPlane, align: NcAlign, rows: NcDim) -> NcIntResu
 /// cursor position.
 ///
 /// The cursor will end at the cell following the last cell output,
-/// just as if ncplane_putc() was called at that spot.
+/// just as if [`ncplane_putc`] was called at that spot.
 ///
 /// Returns the number of cells drawn on success. On error, returns the negative
 /// number of cells drawn.
@@ -374,7 +550,7 @@ pub fn ncplane_hline(plane: &mut NcPlane, cell: &NcCell, len: NcDim) -> NcIntRes
 /// cursor position.
 ///
 /// The cursor will end at the cell following the last cell output,
-/// just as if ncplane_putc() was called at that spot.
+/// just as if [`ncplane_putc`] was called at that spot.
 ///
 /// Returns the number of cells drawn on success. On error, returns the negative
 /// number of cells drawn.
@@ -547,7 +723,7 @@ pub fn ncplane_perimeter_rounded(
 // box -------------------------------------------------------------------------
 
 /// Draws a box with its upper-left corner at the current cursor position,
-/// having dimensions `y_len` * `x_len`.
+/// having dimensions `len_y` * `len_x`.
 ///
 /// The minimum box size is 2x2, and it cannot be drawn off-screen.
 ///
@@ -563,8 +739,8 @@ pub fn ncplane_box_sized(
     lr: &NcCell,
     hline: &NcCell,
     vline: &NcCell,
-    y_len: NcDim,
-    x_len: NcDim,
+    len_y: NcDim,
+    len_x: NcDim,
     boxmask: NcBoxMask,
 ) -> NcIntResult {
     let (mut y, mut x) = (0, 0);
@@ -578,8 +754,8 @@ pub fn ncplane_box_sized(
             lr,
             hline,
             vline,
-            y + y_len as i32 - 1,
-            x + x_len as i32 - 1,
+            y + len_y as i32 - 1,
+            x + len_x as i32 - 1,
             boxmask,
         )
     }
@@ -593,8 +769,8 @@ pub fn ncplane_double_box(
     plane: &mut NcPlane,
     stylemask: NcStyle,
     channels: NcChannels,
-    y_stop: NcDim,
-    x_stop: NcDim,
+    end_y: NcDim,
+    end_x: NcDim,
     boxmask: NcBoxMask,
 ) -> NcIntResult {
     #[allow(unused_assignments)]
@@ -628,8 +804,8 @@ pub fn ncplane_double_box(
                 &lr,
                 &hl,
                 &vl,
-                y_stop as i32,
-                x_stop as i32,
+                end_y as i32,
+                end_x as i32,
                 boxmask,
             );
         }
@@ -652,8 +828,8 @@ pub fn ncplane_double_box_sized(
     plane: &mut NcPlane,
     stylemask: NcStyle,
     channels: NcChannels,
-    y_len: NcDim,
-    x_len: NcDim,
+    len_y: NcDim,
+    len_x: NcDim,
     boxmask: NcBoxMask,
 ) -> NcIntResult {
     let (mut y, mut x) = (0, 0);
@@ -664,8 +840,8 @@ pub fn ncplane_double_box_sized(
         plane,
         stylemask,
         channels,
-        y as NcDim + y_len - 1,
-        x as NcDim + x_len - 1,
+        y as NcDim + len_y - 1,
+        x as NcDim + len_x - 1,
         boxmask,
     )
 }
@@ -678,8 +854,8 @@ pub fn ncplane_rounded_box(
     plane: &mut NcPlane,
     stylemask: NcStyle,
     channels: NcChannels,
-    y_stop: NcDim,
-    x_stop: NcDim,
+    end_y: NcDim,
+    end_x: NcDim,
     boxmask: NcBoxMask,
 ) -> NcIntResult {
     #[allow(unused_assignments)]
@@ -713,8 +889,8 @@ pub fn ncplane_rounded_box(
                 &lr,
                 &hl,
                 &vl,
-                y_stop as i32,
-                x_stop as i32,
+                end_y as i32,
+                end_x as i32,
                 boxmask,
             );
         }
@@ -736,8 +912,8 @@ pub fn ncplane_rounded_box_sized(
     plane: &mut NcPlane,
     stylemask: NcStyle,
     channels: NcChannels,
-    y_len: NcDim,
-    x_len: NcDim,
+    len_y: NcDim,
+    len_x: NcDim,
     boxmask: NcBoxMask,
 ) -> NcIntResult {
     let (mut y, mut x) = (0, 0);
@@ -748,8 +924,8 @@ pub fn ncplane_rounded_box_sized(
         plane,
         stylemask,
         channels,
-        y as NcDim + y_len - 1,
-        x as NcDim + x_len - 1,
+        y as NcDim + len_y - 1,
+        x as NcDim + len_x - 1,
         boxmask,
     )
 }
@@ -757,7 +933,7 @@ pub fn ncplane_rounded_box_sized(
 // gradient --------------------------------------------------------------------
 
 /// Draws a gradient with its upper-left corner at the current cursor position,
-/// stopping at `ystop`×`xstop`.
+/// stopping at `end_y`×`end_x`.
 ///
 /// The glyph composed of `egc` and `stylemask` is used for all cells. The
 /// `NcChannels` specified by `ul`, `ur`, `ll`, and `lr` are composed into
@@ -788,10 +964,10 @@ pub fn ncplane_gradient(
     ur: NcChannels,
     ll: NcChannels,
     lr: NcChannels,
-    y_len: NcDim,
-    x_len: NcDim,
+    len_y: NcDim,
+    len_x: NcDim,
 ) -> NcIntResult {
-    if y_len < 1 || x_len < 1 {
+    if len_y < 1 || len_x < 1 {
         return NCRESULT_ERR;
     }
 
@@ -809,14 +985,14 @@ pub fn ncplane_gradient(
             ur,
             ll,
             lr,
-            y_len as i32,
-            x_len as i32,
+            len_y as i32,
+            len_x as i32,
         )
     }
 }
 
 /// Draw a gradient with its upper-left corner at the current cursor position,
-/// having dimensions `y_len` * `x_len`.
+/// having dimensions `len_y` * `len_x`.
 ///
 /// See [ncplane_gradient][crate::ncplane_gradient] for more information.
 ///
@@ -830,10 +1006,10 @@ pub fn ncplane_gradient_sized(
     ur: NcChannels,
     ll: NcChannels,
     lr: NcChannels,
-    y_len: NcDim,
-    x_len: NcDim,
+    len_y: NcDim,
+    len_x: NcDim,
 ) -> NcIntResult {
-    if y_len < 1 || x_len < 1 {
+    if len_y < 1 || len_x < 1 {
         return NCRESULT_ERR;
     }
     let (mut y, mut x) = (0, 0);
@@ -847,8 +1023,8 @@ pub fn ncplane_gradient_sized(
             ur,
             ll,
             lr,
-            y as u32 + y_len - 1,
-            x as u32 + x_len - 1,
+            y as u32 + len_y - 1,
+            x as u32 + len_x - 1,
         )
     }
 }
