@@ -4,11 +4,14 @@ use core::ptr::null_mut;
 use libc::c_void;
 
 use crate::{
-    cstring, error, error_ref_mut, rstring, NcBlitter, NcDim, NcDirect, NcDirectF, NcDirectV,
-    NcError, NcIntResult, NcPixel, NcPlane, NcResult, NcRgba, NcScale, NcTime, NcVGeom, NcVisual,
-    NcVisualOptions, Notcurses, NCBLIT_PIXEL, NCRESULT_ERR,
+    cstring, error, error_ref_mut, rstring_free, Nc, NcBlitter, NcBlitterGeometry, NcChannel,
+    NcComponent, NcDim, NcDirect, NcDirectF, NcError, NcIntResult, NcPixel, NcPlane, NcResult,
+    NcRgba, NcScale, NcTime, NcVGeom, NcVisual, NcVisualOptions, NCBLIT_PIXEL, NCRESULT_ERR,
 };
 
+#[allow(unused_imports)]
+use crate::{NCBLIT_1x1, NCBLIT_2x1, NCBLIT_2x2, NCBLIT_3x2, NCSCALE_STRETCH};
+
 /// # NcVisualOptions Constructors
 impl NcVisualOptions {
     // /// New NcVisualOptions
@@ -36,10 +39,10 @@ impl NcVisualOptions {
         scale: NcScale,
         y: NcDim,
         x: NcDim,
-        begy: NcDim,
-        begx: NcDim,
-        leny: NcDim,
-        lenx: NcDim,
+        beg_y: NcDim,
+        beg_x: NcDim,
+        len_y: NcDim,
+        len_x: NcDim,
         blitter: NcBlitter,
         flags: u32,
         transcolor: NcRgba,
@@ -52,11 +55,11 @@ impl NcVisualOptions {
             y: y as i32,
             x: x as i32,
             // origin of rendered section
-            begy: begy as i32,
-            begx: begx as i32,
+            begy: beg_y as i32,
+            begx: beg_x as i32,
             // size of rendered section
-            leny: leny as i32,
-            lenx: lenx as i32,
+            leny: len_y as i32,
+            lenx: len_x as i32,
             // glyph set to use
             blitter,
             // bitmask over NCVISUAL_OPTION_*
@@ -68,10 +71,10 @@ impl NcVisualOptions {
     pub fn without_plane(
         y: NcDim,
         x: NcDim,
-        begy: NcDim,
-        begx: NcDim,
-        leny: NcDim,
-        lenx: NcDim,
+        beg_y: NcDim,
+        beg_x: NcDim,
+        len_y: NcDim,
+        len_x: NcDim,
         blitter: NcBlitter,
         flags: u32,
         transcolor: u32,
@@ -83,11 +86,11 @@ impl NcVisualOptions {
             y: y as i32,
             x: x as i32,
             // origin of rendered section
-            begy: begy as i32,
-            begx: begx as i32,
+            begy: beg_y as i32,
+            begx: beg_x as i32,
             // size of rendered section
-            leny: leny as i32,
-            lenx: lenx as i32,
+            leny: len_y as i32,
+            lenx: len_x as i32,
             // glyph set to use
             blitter,
             // bitmask over NCVISUAL_OPTION_*
@@ -97,8 +100,8 @@ impl NcVisualOptions {
         }
     }
 
-    pub fn fullsize_pixel_without_plane(y: NcDim, x: NcDim, leny: NcDim, lenx: NcDim) -> Self {
-        Self::without_plane(y, x, 0, 0, leny, lenx, NCBLIT_PIXEL, 0, 0)
+    pub fn fullsize_pixel_without_plane(y: NcDim, x: NcDim, len_y: NcDim, len_x: NcDim) -> Self {
+        Self::without_plane(y, x, 0, 0, len_y, len_x, NCBLIT_PIXEL, 0, 0)
     }
 }
 
@@ -176,6 +179,64 @@ impl NcVisual {
         ]
     }
 
+    /// Like [`from_rgba`][NcVisual#method.from_rgba], but the pixels are
+    /// 4-byte RGBX. Alpha is filled in throughout using 'alpha'.
+    ///
+    /// `rowstride` must be a multiple of 4.
+    ///
+    /// *C style function: [ncvisual_from_rgb_loose()][crate::ncvisual_from_rgb_loose].*
+    pub fn from_rgb_loose<'a>(
+        rgb: &[u8],
+        rows: NcDim,
+        rowstride: NcDim,
+        cols: NcDim,
+        alpha: NcComponent,
+    ) -> NcResult<&'a mut NcVisual> {
+        error_ref_mut![
+            unsafe {
+                crate::ncvisual_from_rgb_loose(
+                    rgb.as_ptr() as *const c_void,
+                    rows as i32,
+                    rowstride as i32,
+                    cols as i32,
+                    alpha as i32,
+                )
+            },
+            &format!(
+                "NcVisual::from_rgb_loose(rgba, {}, {}, {}, {})",
+                rows, rowstride, cols, alpha
+            )
+        ]
+    }
+
+    /// Like [`from_rgba`][NcVisual#method.from_rgba], but the pixels are
+    /// 3-byte RGB. Alpha is filled in throughout using 'alpha'.
+    ///
+    /// *C style function: [ncvisual_from_rgb_packed()][crate::ncvisual_from_rgb_packed].*
+    pub fn from_rgb_packed<'a>(
+        rgb: &[u8],
+        rows: NcDim,
+        rowstride: NcDim,
+        cols: NcDim,
+        alpha: NcComponent,
+    ) -> NcResult<&'a mut NcVisual> {
+        error_ref_mut![
+            unsafe {
+                crate::ncvisual_from_rgb_packed(
+                    rgb.as_ptr() as *const c_void,
+                    rows as i32,
+                    rowstride as i32,
+                    cols as i32,
+                    alpha as i32,
+                )
+            },
+            &format!(
+                "NcVisual::from_rgb_packed(rgba, {}, {}, {}, {})",
+                rows, rowstride, cols, alpha
+            )
+        ]
+    }
+
     /// Prepares an NcVisual, and its underlying NcPlane, based off RGBA content
     /// in memory at `rgba`.
     ///
@@ -209,6 +270,54 @@ impl NcVisual {
         ]
     }
 
+    /// Like [`from_rgba`][NcVisual#method.from_rgba], but `data` is
+    /// `pstride`-byte palette-indexed pixels, arranged in `rows` lines of
+    /// `rowstride` bytes each, composed of `cols` pixels.
+    ///
+    /// `palette` is an array of at least `palsize` [`NcChannel`]s.
+    ///
+    /// *C style function: [ncvisual_from_palidx()][crate::ncvisual_from_palidx].*
+    //
+    // API ALLOC struct ncvisual* ncvisual_from_palidx(const void* data, int rows,
+    // int rowstride, int cols, int palsize, int pstride, const uint32_t* palette)
+    // pub fn ncvisual_from_palidx(
+    //     data: *const cty::c_void,
+    //     rows: cty::c_int,
+    //     rowstride: cty::c_int,
+    //     cols: cty::c_int,
+    //     palsize: cty::c_int,
+    //     pstride: cty::c_int,
+    //     palette: *const u32,
+    // ) -> *mut ncvisual;
+    pub fn from_palidx<'a>(
+        data: &[u8],
+        rows: NcDim,
+        rowstride: NcDim,
+        cols: NcDim,
+        palsize: u8,
+        pstride: NcDim,
+        palette: &[NcChannel],
+    ) -> NcResult<&'a mut NcVisual> {
+        // assert![];
+        error_ref_mut![
+            unsafe {
+                crate::ncvisual_from_palidx(
+                    data.as_ptr() as *const c_void,
+                    rows as i32,
+                    rowstride as i32,
+                    cols as i32,
+                    palsize as i32,
+                    pstride as i32,
+                    palette.as_ptr() as *const NcChannel,
+                )
+            },
+            &format!(
+                "NcVisual::from_palidx(data, {}, {}, {}, {}, {}, palette)",
+                rows, rowstride, cols, palsize, pstride
+            )
+        ]
+    }
+
     /// Destroys this NcVisual.
     ///
     /// Rendered elements will not be disrupted, but the visual can be neither
@@ -263,18 +372,10 @@ impl NcVisual {
         }
     }
 
-    /// Inflates each pixel in the image to 'scale'x'scale' pixels.
-    ///
-    /// The original color is retained.
-    pub fn inflate(&mut self, scale: u32) -> NcResult<NcIntResult> {
-        let res = unsafe { crate::ncvisual_inflate(self, scale as i32) };
-        error![res, &format!["NcVisual.inflate({})", scale], res]
-    }
-
     /// Gets the size and ratio of NcVisual pixels to output cells along the
-    /// `y→to_y` and `x→to_x` axes.
+    /// vertical and horizontal axes.
     ///
-    /// Returns a tuple with (y, x, to_y, to_x)
+    /// Returns [`NcBlitterGeometry`].
     ///
     /// An NcVisual of `y` by `x` pixels will require
     /// (`y` * `to_y`) by (`x` * `to_x`) cells for full output.
@@ -282,15 +383,12 @@ impl NcVisual {
     /// Errors on invalid blitter in `options`. Scaling is taken into consideration.
     ///
     /// *C style function: [ncvisual_blitter_geom()][crate::ncvisual_blitter_geom].*
-    pub fn geom(
-        &self,
-        nc: &Notcurses,
-        options: &NcVisualOptions,
-    ) -> NcResult<(NcDim, NcDim, NcDim, NcDim)> {
+    pub fn blitter_geom(&self, nc: &Nc, options: &NcVisualOptions) -> NcResult<NcBlitterGeometry> {
         let mut y = 0;
         let mut x = 0;
-        let mut to_y = 0;
-        let mut to_x = 0;
+        let mut scale_y = 0;
+        let mut scale_x = 0;
+        let mut blitter = 0;
 
         let res = unsafe {
             crate::ncvisual_blitter_geom(
@@ -299,30 +397,34 @@ impl NcVisual {
                 options,
                 &mut y,
                 &mut x,
-                &mut to_y,
-                &mut to_x,
-                null_mut(),
+                &mut scale_y,
+                &mut scale_x,
+                &mut blitter,
             )
         };
-        error![
-            res,
-            "NcVisual.geom()",
-            (y as NcDim, x as NcDim, to_y as NcDim, to_x as NcDim)
-        ];
+        let geom = NcBlitterGeometry {
+            x: x as NcDim,
+            y: y as NcDim,
+            scale_y: scale_y as NcDim,
+            scale_x: scale_x as NcDim,
+            blitter,
+        };
+        error![res, "NcVisual.geom()", geom];
     }
 
     /// Gets the default media (not plot) blitter for this environment when using
     /// the specified scaling method.
     ///
     /// Currently, this means:
-    /// - if lacking UTF-8, NCBLIT_1x1.
-    /// - otherwise, if not NCSCALE_STRETCH, NCBLIT_2x1.
-    /// - otherwise, if sextants are not known to be good, NCBLIT_2x2.
-    /// - otherwise NCBLIT_3x2 NCBLIT_2x2 and NCBLIT_3x2 both distort the original
-    ///   aspect ratio, thus NCBLIT_2x1 is used outside of NCSCALE_STRETCH.
+    /// - if lacking UTF-8, [`NCBLIT_1x1`].
+    /// - otherwise, if not [`NCSCALE_STRETCH`], [`NCBLIT_2x1`].
+    /// - otherwise, if sextants are not known to be good, [`NCBLIT_2x2`].
+    /// - otherwise [`NCBLIT_3x2`], [`NCBLIT_2x2`] and [`NCBLIT_3x2`] both
+    ///   distort the original aspect ratio, thus [`NCBLIT_2x1`] is used
+    ///   outside of [`NCSCALE_STRETCH`].
     ///
     /// *C style function: [ncvisual_media_defblitter()][crate::ncvisual_media_defblitter].*
-    pub fn media_defblitter(nc: &Notcurses, scale: NcScale) -> NcBlitter {
+    pub fn media_defblitter(nc: &Nc, scale: NcScale) -> NcBlitter {
         unsafe { crate::ncvisual_media_defblitter(nc, scale) }
     }
 
@@ -336,23 +438,22 @@ impl NcVisual {
         ]
     }
 
-    /// Renders the decoded frame to the specified [NcPlane].
+    /// Renders the decoded frame to the [`NcPlane`] specified in `options`.
+    ///
+    /// If a plane is not provided, it will be created, having the exact size
+    /// necessary to display the visual.
     ///
     /// See [`NcVisualOptions`].
     ///
     /// *C style function: [ncvisual_render()][crate::ncvisual_render].*
-    pub fn render(
-        &mut self,
-        nc: &mut Notcurses,
-        options: &NcVisualOptions,
-    ) -> NcResult<&mut NcPlane> {
+    pub fn render(&mut self, nc: &mut Nc, options: &NcVisualOptions) -> NcResult<&mut NcPlane> {
         error_ref_mut![
             unsafe { crate::ncvisual_render(nc, self, options) },
-            "NcVisual.render(Notcurses, &NcVisualOptions)"
+            "NcVisual.render(Nc, &NcVisualOptions)"
         ]
     }
 
-    /// Resizes the visual to `rows` X `columns` pixels.
+    /// Resizes the visual to `cols` X `rows` pixels.
     ///
     /// This is a lossy transformation, unless the size is unchanged.
     ///
@@ -364,6 +465,20 @@ impl NcVisual {
         ]
     }
 
+    /// Resizes the visual to  in the image to `rows` X `cols` pixels, without
+    /// interpolating the color values.
+    ///
+    /// The original color is retained.
+    ///
+    /// *C style function:
+    /// [ncvisual_resize_noninterpolative()][crate::ncvisual_resize_noninterpolative].*
+    pub fn resize_noninterpolative(&mut self, rows: NcDim, cols: NcDim) -> NcResult<()> {
+        error![
+            unsafe { crate::ncvisual_resize_noninterpolative(self, rows as i32, cols as i32) },
+            &format!["NcVisual.resize_noninterpolative({}, {})", cols, rows]
+        ]
+    }
+
     /// Rotates the visual `rads` radians.
     ///
     /// Only M_PI/2 and -M_PI/2 are supported at the moment,
@@ -448,7 +563,7 @@ impl NcVisual {
     // //
     // pub fn simple_streamer(
     //     &mut self,
-    //     nc: &mut Notcurses,
+    //     nc: &mut Nc,
     //     timescale: f32,
     //     //streamer: Option<streamcb>
     //     options: &NcVisualOptions,
@@ -460,14 +575,25 @@ impl NcVisual {
     /// copy of the UTF8 text.
     ///
     /// *C style function: [ncvisual_subtitle()][crate::ncvisual_subtitle].*
-    pub fn subtitle(&self) -> NcResult<&str> {
+    #[deprecated]
+    pub fn subtitle(&self) -> NcResult<String> {
         let res = unsafe { crate::ncvisual_subtitle(self) };
         if !res.is_null() {
-            Ok(rstring![res])
+            Ok(rstring_free![res])
         } else {
             Err(NcError::with_msg(NCRESULT_ERR, "NcVisual.subtitle()"))
         }
     }
+
+    /// If a subtitle ought be displayed at this time, return a new plane
+    ///
+    /// The returned plane is bound to `parent` and contains the subtitle,
+    /// which might be text or graphics (depending on the input format).
+    ///
+    /// *C style function: [ncvisual_subtitle_plane()][crate::ncvisual_subtitle_plane].*
+    pub fn subtitle_plane(&self, parent: &mut NcPlane) -> NcResult<&mut NcPlane> {
+        error_ref_mut![unsafe { crate::ncvisual_subtitle_plane(parent, self) }]
+    }
 }
 
 /// # NcDirectF Constructors & destructors
@@ -514,7 +640,7 @@ impl NcDirectF {
         &mut self,
         ncd: &mut NcDirect,
         options: &NcVisualOptions,
-    ) -> NcResult<&mut NcDirectV> {
+    ) -> NcResult<&mut NcPlane> {
         error_ref_mut![
             unsafe { crate::ncdirectf_render(ncd, self, options) },
             "NcVisual.render()"
