@@ -1,6 +1,6 @@
-// functions already exported by bindgen : 24
+// functions already exported by bindgen : 27
 // -----------------------------------------
-// (W) wrap: 20
+// (W) wrap: 23
 // (#) test: 0
 // -----------------------------------------
 //W  ncdirectf_free
@@ -10,13 +10,14 @@
 //W  ncvisual_at_yx
 //W  ncvisual_decode
 //W  ncvisual_decode_loop
-//   ncvisual_default_blitter
 //W  ncvisual_destroy
 //W  ncvisual_from_bgra
 //W  ncvisual_from_file
+//W  ncvisual_from_palidx
 //W  ncvisual_from_plane
 //W  ncvisual_from_rgba
-//W  ncvisual_inflate
+//W  ncvisual_from_rgb_packed
+//W  ncvisual_from_rgb_loose
 //W  ncvisual_blitter_geom
 //W  ncvisual_media_defblitter
 //W  ncvisual_polyfill_yx
@@ -28,13 +29,14 @@
 //W  ncvisual_simple_streamer
 //~  ncvisual_stream
 //W  ncvisual_subtitle
+//W  ncvisual_subtitle_plane
 
 #[allow(unused_imports)] // for the doc comments
 use crate::{NcChannel, NcDim, NcRgb};
 
 mod methods;
 
-/// How to scale an [`NcVisual`] during rendering.
+/// Indicates how to scale an [`NcVisual`] during rendering.
 ///
 /// - [`NCSCALE_NONE`] will apply no scaling.
 /// - [`NCSCALE_SCALE`] scales a visual to the plane's size,
@@ -44,6 +46,18 @@ mod methods;
 /// - [`NCSCALE_NONE_HIRES`] like `NCSCALE_NONE` admitting high-res blitters.
 /// - [`NCSCALE_SCALE_HIRES`] like `NCSCALE_SCALE` admitting high-res blitters.
 ///
+/// The `NCSCALE_*` preferences are applied only for the context of
+/// [`NcVisual.render`][NcVisual#method.render]. You can think of it as a pipeline:
+///
+/// ```txt
+/// NcVisual::fromfile() → frame → NcVisual.render() → scaling → output frame → blit
+/// ```
+///
+/// where you still have the original frame. Whereas
+/// [`NcVisual.resize`][NcVisual#method.resize] and
+/// [`NcVisual.resize_noninterpolative`][NcVisual#method.resize_noninterpolative]
+/// are changing that original frame.
+///
 pub type NcScale = crate::bindings::ffi::ncscale_e;
 
 /// Maintains original size.
@@ -68,9 +82,9 @@ pub const NCSCALE_SCALE_HIRES: NcScale = crate::bindings::ffi::ncscale_e_NCSCALE
 /// It can be constructed from a rgba or bgra buffer.
 ///
 /// The [NcVisualOptions] structure is used only by the following methods:
-/// - [.geom][NcVisual#method.render]
+/// - [.geom][NcVisual#method.geom]
 /// - [.render][NcVisual#method.render]
-/// - [.simple_streamer][NcVisual#method.render]
+/// - [.simple_streamer][NcVisual#method.simple_streamer]
 pub type NcVisual = crate::bindings::ffi::ncvisual;
 
 /// A type alias of [`NcVisual`] (NcDirect ***F**rame*) intended to be used
@@ -94,10 +108,10 @@ pub type NcVGeom = crate::bindings::ffi::ncvgeom;
 ///
 /// If a plane is not provided, one will be created, having the exact size
 /// necessary to display the visual (this might be smaller or larger than
-/// the rendering area). if NCVISUAL_OPTION_CHILDPLANE is provided, this
+/// the rendering area). if [`NCVISUAL_OPTION_CHILDPLANE`] is provided, this
 /// will be interpreted as the parent.
 ///
-/// A subregion of the visual can be rendered using `begx`, `begy`, `lenx`, and `leny`.
+/// A subregion of the visual can be rendered using `beg_x`, `beg_y`, `len_x`, and `len_y`.
 pub type NcVisualOptions = crate::bindings::ffi::ncvisual_options;
 
 // NcRgba
@@ -134,7 +148,7 @@ pub type NcRgba = u32;
 /// Treats as transparent the color specified in the `transcolor` field.
 pub const NCVISUAL_OPTION_ADDALPHA: u32 = crate::bindings::ffi::NCVISUAL_OPTION_ADDALPHA;
 
-/// Uses [`NCCELL_ALPHA_BLEND`][crate::NCCELL_ALPHA_BLEND] with visual.
+/// Uses [`NCALPHA_BLEND`][crate::NCALPHA_BLEND] with visual.
 pub const NCVISUAL_OPTION_BLEND: u32 = crate::bindings::ffi::NCVISUAL_OPTION_BLEND;
 
 /// allows you to indicate that the n field of ncvisual_options refers not to
@@ -145,7 +159,7 @@ pub const NCVISUAL_OPTION_BLEND: u32 = crate::bindings::ffi::NCVISUAL_OPTION_BLE
 /// a sprixel relative to the standard plane can be done in one step.
 pub const NCVISUAL_OPTION_CHILDPLANE: u32 = crate::bindings::ffi::NCVISUAL_OPTION_CHILDPLANE;
 
-/// Fails rather than gracefully degrade. See [NcBlitter].
+/// Fails rather than gracefully degrade. See [`NcBlitter`].
 pub const NCVISUAL_OPTION_NODEGRADE: u32 = crate::bindings::ffi::NCVISUAL_OPTION_NODEGRADE;
 
 /// Y is an alignment, not absolute.
@@ -228,11 +242,37 @@ pub const NCBLIT_PIXEL: NcBlitter = crate::bindings::ffi::ncblitter_e_NCBLIT_PIX
 /// If bitmaps are not supported, the fields `max_bitmap_*` will be 0.
 #[derive(Clone, Debug)]
 pub struct NcPixelGeometry {
-    /// Geometry of the display region
-    pub display_y: NcDim,
-    pub display_x: NcDim,
+    /// The height in pixels of the display region.
+    pub term_y: NcDim,
+    /// The width in pixels of the display region.
+    pub term_x: NcDim,
+    /// The height in pixels of a single cell.
     pub cell_y: NcDim,
+    /// The width in pixels of a single cell.
     pub cell_x: NcDim,
+    /// The height in pixels of the maximum displayable bitmap (0 if not supported).
     pub max_bitmap_y: NcDim,
+    /// The width in pixels of the maximum displayable bitmap (0 if not supported).
     pub max_bitmap_x: NcDim,
 }
+
+/// Contains the blitter geometry information as returned by the
+/// NcPlane.[blitter_geom()][crate::NcPlane#method.blitter_geom] method.
+///
+/// - `y`, `x`: the input size in pixels.
+/// - `scale_y`, `scale_x`: the scaling
+/// - `blitter` The blitter that will be used
+///
+#[derive(Clone, Debug)]
+pub struct NcBlitterGeometry {
+    ///
+    y: NcDim,
+    ///
+    x: NcDim,
+    ///
+    scale_y: NcDim,
+    ///
+    scale_x: NcDim,
+    /// The blitter that will be used.
+    blitter: NcBlitter,
+}
