@@ -7,6 +7,7 @@ mod progbar;
 mod reader;
 mod reel;
 mod selector;
+mod tabbed;
 mod tree;
 
 pub use menu::*;
@@ -16,4 +17,5 @@ pub use progbar::*;
 pub use reader::*;
 pub use reel::*;
 pub use selector::*;
+pub use tabbed::*;
 pub use tree::*;
